==> ttetris-0.1.tar.gz: ttetris-0.1/score.c <==
/*****************************************************************
 Project Name: Terminal Tetris
 File Name: score.c
 Description:  Լ.

 Created: 2004. 02. 22
 RCS-ID: $Id: score.c,v 1.12 2004/06/11 05:30:13 myunggoni Exp $
 Copyright: 2004 (c) by myunggoni
 License: GNI General Public License
 Author: Myung-Gon, Park <myunggoni@users.kldp.net>
 ******************************************************************/

#include <ncurses.h>
#include <stdlib.h>
#include <string.h>
#include "main.h"
#include "config.h"

static high_score score_list[MAX_SCORE_LIST];

int get_score_list() {
	FILE *fp;
	int temp;
	int i = 0;
	char buf[MAX_NAME];

	if((fp = fopen(SCORE_DATA, "r")) != NULL) {

		for(i = 0; i < MAX_SCORE_LIST; i++) {
			if(fscanf(fp, "%d\n", &temp) != EOF)
				score_list[i].score = temp;
			else
				break;
			
			fscanf(fp, "%d\n", &temp);
			score_list[i].level = temp;
			
			fgets(buf, MAX_NAME, fp);
			strncpy(score_list[i].name, buf, MAX_NAME);
		}
		fclose(fp);
	}

	if(i < MAX_SCORE_LIST) {
		while(i < MAX_SCORE_LIST) {
			score_list[i].score = 0;
			score_list[i].level = 0;
			strncpy(score_list[i].name, "No Name", MAX_NAME);
			i++;
		}
	}
	return 0;
}

int check_score_list(int score, int level) { 
	int i, j;
	high_score user_info;
	
	get_score_list(); 
	
	for(i = 0; i < MAX_SCORE_LIST; i++) {
		if(score_list[i].score < score) {
			user_info = get_username(score, level);
			break;
		} else if(score_list[i].score == score) {
				if(score_list[i].level <= level) {
					user_info = get_username(score, level);
					break;
				}				
		}
	}

	if(i != MAX_SCORE_LIST) {
		for(j = MAX_SCORE_LIST - 1; j > i; j--) {
			score_list[j] = score_list[j - 1];			
		}
		
		score_list[i] = user_info;
		put_score_list();
		return 0; /*  Էµ. */
	}

	return 1; /* ְ  Ʈ  . */
}	

high_score get_username(int score, int level) { 
	int ch;
	int i;
	char name[MAX_NAME + 1];
	static high_score user_info;
	
	do {
		clear();
		mvprintw(MENU_Y, MENU_X, "[New Record]");
		
		mvprintw(MENU_Y + 2, MENU_X, "Enter Your Name(Max 12 Characters): ");
		
		memset(name, 0, sizeof(name));

		for(i = 0; i < MAX_NAME; i++) {
			if((ch = getch()) == '\n') {
				name[i] = '\0';
				break;

			}
			name[i] = ch;
		}

		if(name[0] != '\0') {
			mvprintw(MENU_Y + 4, MENU_X, "You Entered \"%s\"", name);
			mvprintw(MENU_Y + 6, MENU_X, "OK?(y/n): ");

			ch = getch();

			while(ch != 'y' && ch != 'Y' && ch != 'n' && ch != 'N') {
				mvhline(MENU_Y + 6, MENU_X, ' ', 20);
				mvprintw(MENU_Y + 6, MENU_X, "Wrong Input. \'y\' or \'n\': ");

				ch = getch();
			}

		} else {
			break;
		}
		
	} while(ch == 'n' || ch == 'N');

	user_info.score = score; 
	user_info.level = level;
	
	if(name[0] == '\0')
		strncpy(user_info.name, "Anonymous", MAX_NAME);
	else
		strncpy(user_info.name, name, MAX_NAME);
	
	return user_info; 
}

int put_score_list() { 
	FILE *fp;
	int i;
	
	if((fp = fopen(SCORE_DATA, "w")) == NULL) 
		error_handling("fopen");
	

	for(i = 0; i < MAX_SCORE_LIST; i++) {
		fprintf(fp, "%d\n", score_list[i].score);
		fprintf(fp, "%d\n", score_list[i].level);
		fprintf(fp, "%s\n", score_list[i].name);
	}
	fclose(fp);
	
	return 0;
}

void show_score_list() {
	int i;
	
	clear();
	mvprintw(MENU_Y, MENU_X, "[High Score]");
	mvprintw(MENU_Y + 2, MENU_X, "%3s%12s%12s%14s", "No.", "Score", "Level", "Name");
	mvhline(MENU_Y + 3, MENU_X, '-', 41);
	get_score_list();

	for(i = 0; i < MAX_SCORE_LIST; i++) {
		mvprintw(MENU_Y + i + 4, MENU_X, "%2d %12d%12d%14s", i + 1, score_list[i].score, score_list[i].level, score_list[i].name);
	}

	mvprintw(MENU_Y + i + 5, MENU_X, "Press Any Key...");
	getch();
}

==> ttetris-0.1.4.tar.gz: ttetris-0.1.4/score.c <==
/*****************************************************************
 Project Name: Terminal Tetris
 File Name: score.c
 Description:  Լ.

 Created: 2004. 02. 22
 RCS-ID: $Id: score.c,v 1.19 2004/06/18 06:28:14 myunggoni Exp $
 Copyright: (c) 2004 by myunggoni
 License: GNI General Public License
 Author: Myung-Gon, Park <myunggoni@users.kldp.net>
 ******************************************************************/

#include <ncurses.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include "main.h"
#include "config.h"

#define writew_lock(fd, offset, whence, len) \
	lock_reg(fd, F_SETLKW, F_WRLCK, offset, whence, len)

static high_score score_list[MAX_SCORE_LIST];

int get_score_list() {
	FILE *fp;
	int temp;
	int i = 0;
	char buf[MAX_NAME];

	if((fp = fopen(SCORE_DATA, "r")) != NULL) {

		for(i = 0; i < MAX_SCORE_LIST; i++) {
			if(fscanf(fp, "%d\n", &temp) != EOF)
				score_list[i].score = temp;
			else
				break;
			
			fscanf(fp, "%d\n", &temp);
			score_list[i].level = temp;
			
			fgets(buf, MAX_NAME, fp);
			strncpy(score_list[i].name, buf, MAX_NAME);
		}
		fclose(fp);
	}

	if(i < MAX_SCORE_LIST) {
		while(i < MAX_SCORE_LIST) {
			score_list[i].score = 0;
			score_list[i].level = 0;
			strncpy(score_list[i].name, "No Name", MAX_NAME);
			i++;
		}
	}
	
	return 0;
}

int check_score_list(int score, int level) { 
	int i, j;
	
	get_score_list(); 
	
	for(i = 0; i < MAX_SCORE_LIST; i++) {
		if(score_list[i].score < score) 
			break;
		else if(score_list[i].score == score) 
				if(score_list[i].level <= level) 
					break;
	}
	
	if(i != MAX_SCORE_LIST)
		return i; /*  . */
	else 
		return -1; /* ְ  Ʈ  . */
}	

high_score get_username(int score, int level) { 
	int ch;
	int i;
	char name[MAX_NAME + 1];
	high_score user_info;
	
	do {
		clear();
		mvprintw(MENU_Y, MENU_X, "[New Record]");
		
		mvprintw(MENU_Y + 2, MENU_X, "Enter Your Name(Max 12 Characters): ");
		
		memset(name, 0, sizeof(name));

		for(i = 0; i < MAX_NAME; i++) {
			if((ch = getch()) == '\n') {
				name[i] = '\0';
				break;

			}
			name[i] = ch;
		}

		if(name[0] != '\0') {
			mvprintw(MENU_Y + 4, MENU_X, "You Entered \"%s\". OK?(y/n): ", name);

			ch = getch();

			while(ch != 'y' && ch != 'Y' && ch != 'n' && ch != 'N') {
				mvhline(MENU_Y + 4, MENU_X, ' ', 40);
				mvprintw(MENU_Y + 4, MENU_X, "Wrong Input. \'y\' or \'n\': ");

				ch = getch();
			}

		} else {
			break;
		}
		
	} while(ch == 'n' || ch == 'N');

	user_info.score = score; 
	user_info.level = level;
	
	if(name[0] == '\0')
		strncpy(user_info.name, "Anonymous", MAX_NAME);
	else
		strncpy(user_info.name, name, MAX_NAME);

	return user_info; 
}

int put_score_list(int rank, int score, int level) { 
	FILE *fp;
	int i, j;
	int flag = 0;
	high_score user_info;
	high_score score_list_tmp[MAX_SCORE_LIST];
	
	user_info = get_username(score, level);
	
	for(i = 0; i < MAX_SCORE_LIST; i++) {
		score_list_tmp[i].score = score_list[i].score;
		score_list_tmp[i].level = score_list[i].score;
		strncpy(score_list_tmp[i].name, score_list[i].name, MAX_NAME);
	}

	get_score_list();
	
	for(i = 0; i < MAX_SCORE_LIST; i++) {
		if(score_list_tmp[i].score != score_list[i].score) {
			flag = 1;
			break;
		}
			
		if(score_list_tmp[i].level != score_list[i].score) {
			flag = 1;
			break;
		}
			
		if(strncmp(score_list_tmp[i].name, score_list[i].name, MAX_NAME) != 0) {
			flag = 1;
			break;
		}
	}

	if(flag) {
		if((rank = check_score_list(score, level)) == -1)
			return 0;
	}
	
	if((fp = fopen(SCORE_DATA, "w")) == NULL) 
		error_handling("fopen");
	
	if(writew_lock(fileno(fp), 0, SEEK_SET, 0) < 0) 
			error_handling("fcntl");

	for(j = MAX_SCORE_LIST - 1; j > rank; j--) {
			score_list[j] = score_list[j - 1];			
	}
		
	score_list[rank] = user_info;
	
	
	for(i = 0; i < MAX_SCORE_LIST; i++) {
		fprintf(fp, "%d\n", score_list[i].score);
		fprintf(fp, "%d\n", score_list[i].level);
		fprintf(fp, "%s\n", score_list[i].name);
	}
	
	fclose(fp);
	
	return 0;
}

void show_score_list() {
	int i;
	
	clear();
	mvprintw(MENU_Y, MENU_X, "[High Score]");
	mvprintw(MENU_Y + 2, MENU_X, "%3s%12s%12s%14s", "No.", "Score", "Level", "Name");
	mvhline(MENU_Y + 3, MENU_X, '-', 41);
	get_score_list();

	for(i = 0; i < MAX_SCORE_LIST; i++) {
		mvprintw(MENU_Y + i + 4, MENU_X, "%2d %12d%12d%14s", i + 1, score_list[i].score, score_list[i].level, score_list[i].name);
	}

	mvprintw(MENU_Y + i + 5, MENU_X, "Press Any Key...");
	getch();
}

int lock_reg(int fd, int cmd, int type, off_t offset, int whence, off_t len) {
	struct flock lock;

	lock.l_type = type;
	lock.l_start = offset;
	lock.l_whence = whence;
	lock.l_len = len;

	return(fcntl(fd, cmd, &lock));
}
